<?php
/**
 * @link https://github.com/borodulin/yii2-oauth2-server
 * @copyright Copyright (c) 2015 Andrey Borodulin
 * @license https://github.com/borodulin/yii2-oauth2-server/blob/master/LICENSE
 */

namespace yuncms\oauth2\models;

use Yii;
use yii\db\ActiveRecord;
use yii\helpers\VarDumper;
use yuncms\oauth2\Exception;
use yuncms\user\models\User;

/**
 * This is the model class for table "oauth_authorization_code".
 *
 * @property string $authorization_code
 * @property string $client_id
 * @property integer $user_id
 * @property string $redirect_uri
 * @property integer $expires
 * @property string $scope
 *
 * @property Client $client
 * @property User $user
 */
class AuthorizationCode extends ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%oauth2_authorization_code}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['authorization_code', 'client_id', 'user_id', 'expires'], 'required'],
            [['user_id', 'expires'], 'integer'],
            [['scope'], 'string'],
            [['authorization_code'], 'string', 'max' => 40],
            [['client_id'], 'string', 'max' => 80],
            [['redirect_uri'], 'url'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'authorization_code' => Yii::t('oauth2', 'Authorization Code'),
            'client_id' => Yii::t('oauth2', 'Client ID'),
            'user_id' => Yii::t('oauth2', 'User ID'),
            'redirect_uri' => Yii::t('oauth2', 'Redirect Uri'),
            'expires' => Yii::t('oauth2', 'Expires'),
            'scope' => Yii::t('oauth2', 'Scopes'),
        ];
    }

    /**
     *
     * @param array $params
     * @throws Exception
     * @return \yuncms\oauth2\models\AuthorizationCode
     */
    public static function createAuthorizationCode(array $params)
    {
        static::deleteAll(['<', 'expires', time()]);
        $params['authorization_code'] = Yii::$app->security->generateRandomString(40);
        $authCode = new static($params);

        if ($authCode->save()) {
            return $authCode;
        } else {
            Yii::error(__CLASS__ . ' validation error: ' . VarDumper::dumpAsString($authCode->errors));
        }
        throw new Exception(Yii::t('oauth2', 'Unable to create authorization code'), Exception::SERVER_ERROR);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getClient()
    {
        return $this->hasOne(Client::className(), ['client_id' => 'client_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(User::className(), ['id' => 'user_id']);
    }
}